@@ -19,13 +19,14 @@
 #include <linux/string.h>
 #include <linux/stat.h>
 #include <linux/locks.h>
+#include <linux/errno.h>
 
 #include <asm/system.h>
 #include <asm/segment.h>
 
 void ext2_put_inode (struct inode * inode)
 {
-	if (inode->i_nlink)
+	if (inode->i_nlink || inode->i_ino == EXT2_ACL_INO)
 		return;
 	inode->i_size = 0;
 	if (inode->i_blocks)
@@ -69,6 +70,8 @@ static struct super_operations ext2_sops = {
 	ext2_statfs
 };
 
+#ifdef EXT2FS_PRE_02B_COMPAT
+
 static int convert_pre_02b_fs (struct super_block * sb,
 			       struct buffer_head * bh)
 {
@@ -107,6 +110,8 @@ static int convert_pre_02b_fs (struct super_block * sb,
 	return 1;
 }
 
+#endif
+
 struct super_block * ext2_read_super (struct super_block * s, void * data,
 				      int silent)
 {
@@ -115,7 +120,9 @@ struct super_block * ext2_read_super (struct super_block * s, void * data,
 	int dev = s->s_dev;
 	int bh_count;
 	int i, j;
+#ifdef EXT2FS_PRE_02B_COMPAT
 	int fs_converted = 0;
+#endif
 
 	lock_super (s);
 	if (!(bh = bread (dev, 1, BLOCK_SIZE))) {
@@ -151,6 +158,7 @@ struct super_block * ext2_read_super (struct super_block * s, void * data,
 	s->u.ext2_sb.s_was_mounted_valid = es->s_valid;
 	s->u.ext2_sb.s_rename_lock = 0;
 	s->u.ext2_sb.s_rename_wait = NULL;
+#ifdef EXT2FS_PRE_02B_COMPAT
 	if (s->s_magic == EXT2_OLD_SUPER_MAGIC) {
 		if (es->s_blocks_count > 262144) {
 			 /* fs > 256 MB can't be converted */ 
@@ -180,6 +188,7 @@ struct super_block * ext2_read_super (struct super_block * s, void * data,
 		printk ("EXT2-fs: conversion succeeded !!!\n");
 		fs_converted = 1;
 	}
+#endif
 	if (s->s_magic != EXT2_SUPER_MAGIC) {
 		s->s_dev = 0;
 		unlock_super (s);
@@ -201,8 +210,8 @@ struct super_block * ext2_read_super (struct super_block * s, void * data,
 			"running e2fsck is recommended\n");
 	s->u.ext2_sb.s_groups_count = (s->u.ext2_sb.s_blocks_count -
 				       s->u.ext2_sb.s_first_data_block +
-				       (EXT2_BLOCK_SIZE(s) * 8) - 1) /
-				       (EXT2_BLOCK_SIZE(s) * 8);
+				       EXT2_BLOCKS_PER_GROUP(s) - 1) /
+				       EXT2_BLOCKS_PER_GROUP(s);
 	for (i = 0; i < EXT2_MAX_GROUP_DESC; i++)
 		s->u.ext2_sb.s_group_desc[i] = NULL;
 	bh_count = (s->u.ext2_sb.s_groups_count +
@@ -247,11 +256,13 @@ struct super_block * ext2_read_super (struct super_block * s, void * data,
 		bh->b_dirt = 1;
 		s->s_dirt = 1;
 	}
+#ifdef EXT2FS_PRE_02B_COMPAT
 	if (fs_converted) {
 		for (i = 0; i < bh_count; i++)
 			s->u.ext2_sb.s_group_desc[i]->b_dirt = 1;
 		s->s_dirt = 1;
 	}
+#endif
 	printk ("[EXT II FS %s, bs=%d, fs=%d, gc=%d, bpg=%d, ipg=%d]\n",
 		EXT2FS_VERSION, s->s_blocksize, s->u.ext2_sb.s_frag_size,
 		s->u.ext2_sb.s_groups_count,
@@ -412,7 +423,7 @@ int ext2_bmap (struct inode * inode, int block)
 }
 
 static struct buffer_head * inode_getblk (struct inode * inode, int nr,
-					  int create, int new_block)
+					  int create, int new_block, int *err)
 {
 	int tmp, goal = 0;
 	unsigned long * p;
@@ -431,8 +442,10 @@ repeat:
 	}
 	if (!create || new_block >= 
 	    (current->rlim[RLIMIT_FSIZE].rlim_cur >>
-	     EXT2_BLOCK_SIZE_BITS(inode->i_sb)))
+	     EXT2_BLOCK_SIZE_BITS(inode->i_sb))) {
+		*err = -EFBIG;
 		return NULL;
+	}
 	if (inode->u.ext2_i.i_next_alloc_block == new_block)
 		goal = inode->u.ext2_i.i_next_alloc_goal;
 #ifdef EXT2FS_DEBUG
@@ -474,9 +487,10 @@ repeat:
 
 static struct buffer_head * block_getblk (struct inode * inode,
 					  struct buffer_head * bh, int nr,
-					  int create, int blocksize, int new_block)
+					  int create, int blocksize, 
+					  int new_block, int *err)
 {
-	int tmp, goal;
+	int tmp, goal = 0;
 	unsigned long * p;
 	struct buffer_head * result;
 	int blocks = inode->i_sb->s_blocksize / 512;
@@ -507,14 +521,15 @@ repeat:
 	    (current->rlim[RLIMIT_FSIZE].rlim_cur >> 
 	     EXT2_BLOCK_SIZE_BITS(inode->i_sb))) {
 		brelse (bh);
+		*err = -EFBIG;
 		return NULL;
 	}
 	if (inode->u.ext2_i.i_next_alloc_block == new_block)
 		goal = inode->u.ext2_i.i_next_alloc_goal;
 	if (!goal) {
 		for (tmp = nr-1; tmp>=0; tmp--) {
-			if (bh->b_data[tmp]) {
-				goal = bh->b_data[tmp];
+			if (((unsigned long *) bh->b_data)[tmp]) {
+				goal = ((unsigned long *)bh->b_data)[tmp];
 				break;
 			}
 		}
@@ -547,12 +562,13 @@ repeat:
 }
 
 struct buffer_head * ext2_getblk (struct inode * inode, int block,
-				  int create)
+				  int create, int *err)
 {
 	struct buffer_head * bh;
 	int b;
 	unsigned long addr_per_block = EXT2_ADDR_PER_BLOCK(inode->i_sb);
 
+	*err = -EIO;
 	if (block < 0) {
 		printk ("ext2_getblk: block < 0\n");
 		return NULL;
@@ -564,8 +580,8 @@ struct buffer_head * ext2_getblk (struct inode * inode, int block,
 		return NULL;
 	}
 	/* If this is a sequential block allocation, set the next_alloc_block
-	   to this block now so that all the indblock and data block allocations
-	   use the same goal zone */
+	   to this block now so that all the indblock and data block
+	   allocations use the same goal zone */
 #ifdef EXT2FS_DEBUG
 	printk ("ext2_getblk: block %d, next %d, goal %d.\n", block, 
 		inode->u.ext2_i.i_next_alloc_block,
@@ -576,38 +592,40 @@ struct buffer_head * ext2_getblk (struct inode * inode, int block,
 		inode->u.ext2_i.i_next_alloc_goal++;
 	}
 
+	*err = -ENOSPC;
 	b = block;
 	if (block < EXT2_NDIR_BLOCKS)
-		return inode_getblk (inode, block, create, b);
+		return inode_getblk (inode, block, create, b, err);
 	block -= EXT2_NDIR_BLOCKS;
 	if (block < addr_per_block) {
-		bh = inode_getblk (inode, EXT2_IND_BLOCK, create, b);
+		bh = inode_getblk (inode, EXT2_IND_BLOCK, create, b, err);
 		return block_getblk (inode, bh, block, create,
-				     inode->i_sb->s_blocksize, b);
+				     inode->i_sb->s_blocksize, b, err);
 	}
 	block -= addr_per_block;
 	if (block < addr_per_block * addr_per_block) {
-		bh = inode_getblk (inode, EXT2_DIND_BLOCK, create,b );
+		bh = inode_getblk (inode, EXT2_DIND_BLOCK, create, b, err);
 		bh = block_getblk (inode, bh, block / addr_per_block, create,
-				   inode->i_sb->s_blocksize, b);
+				   inode->i_sb->s_blocksize, b, err);
 		return block_getblk (inode, bh, block & (addr_per_block - 1),
-				     create, inode->i_sb->s_blocksize, b);
+				     create, inode->i_sb->s_blocksize, b, err);
 	}
 	block -= addr_per_block * addr_per_block;
-	bh = inode_getblk (inode, EXT2_TIND_BLOCK, create, b);
-	bh = block_getblk (inode, bh, block / (addr_per_block * addr_per_block),
-			   create, inode->i_sb->s_blocksize, b);
-	bh = block_getblk (inode, bh, (block / addr_per_block) & (addr_per_block - 1),
-			   create, inode->i_sb->s_blocksize, b);
+	bh = inode_getblk (inode, EXT2_TIND_BLOCK, create, b, err);
+	bh = block_getblk (inode, bh, block/(addr_per_block * addr_per_block),
+			   create, inode->i_sb->s_blocksize, b, err);
+	bh = block_getblk (inode, bh, (block/addr_per_block) & (addr_per_block - 1),
+			   create, inode->i_sb->s_blocksize, b, err);
 	return block_getblk (inode, bh, block & (addr_per_block - 1), create,
-			     inode->i_sb->s_blocksize, b);
+			     inode->i_sb->s_blocksize, b, err);
 }
 
-struct buffer_head * ext2_bread (struct inode * inode, int block, int create)
+struct buffer_head * ext2_bread (struct inode * inode, int block, 
+				 int create, int *err)
 {
 	struct buffer_head * bh;
 
-	bh = ext2_getblk (inode, block, create);
+	bh = ext2_getblk (inode, block, create, err);
 	if (!bh || bh->b_uptodate)
 		return bh;
 	ll_rw_block (READ, 1, &bh);
@@ -615,6 +633,7 @@ struct buffer_head * ext2_bread (struct inode * inode, int block, int create)
 	if (bh->b_uptodate)
 		return bh;
 	brelse (bh);
+	*err = -EIO;
 	return NULL;
 }
 
@@ -628,7 +647,8 @@ void ext2_read_inode (struct inode * inode)
 	unsigned long block;
 	struct ext2_group_desc * gdp;
 
-	if ((inode->i_ino != EXT2_ROOT_INO && inode->i_ino < EXT2_FIRST_INO) ||
+	if ((inode->i_ino != EXT2_ROOT_INO && inode->i_ino != EXT2_ACL_INO &&
+	     inode->i_ino < EXT2_FIRST_INO) ||
 	    inode->i_ino > inode->i_sb->u.ext2_sb.s_inodes_count) {
 		printk ("ext2_read_inode: bad inode number of dev %0x04: %d\n",
 			inode->i_dev, inode->i_ino);
@@ -658,29 +678,28 @@ void ext2_read_inode (struct inode * inode)
 	inode->i_atime = raw_inode->i_atime;
 	inode->i_ctime = raw_inode->i_ctime;
 	inode->i_mtime = raw_inode->i_mtime;
-	if (!S_ISFIFO(inode->i_mode))
-		inode->u.ext2_i.i_dtime = raw_inode->i_dtime;
+	inode->u.ext2_i.i_dtime = raw_inode->i_dtime;
 	inode->i_blksize = inode->i_sb->s_blocksize;
 	inode->i_blocks = raw_inode->i_blocks;
-	if (!S_ISFIFO(inode->i_mode)) {
-		inode->u.ext2_i.i_flags = raw_inode->i_flags;
-		inode->u.ext2_i.i_faddr = raw_inode->i_faddr;
-		inode->u.ext2_i.i_frag = raw_inode->i_frag;
-		inode->u.ext2_i.i_fsize = raw_inode->i_fsize;
-		inode->u.ext2_i.i_file_acl = raw_inode->i_file_acl;
-		inode->u.ext2_i.i_dir_acl = raw_inode->i_dir_acl;
-		inode->u.ext2_i.i_version = raw_inode->i_version;
-		inode->u.ext2_i.i_block_group = block_group;
-		inode->u.ext2_i.i_next_alloc_block = 0;
-		inode->u.ext2_i.i_next_alloc_goal = 0;
-		if (S_ISCHR(inode->i_mode) || S_ISBLK(inode->i_mode))
-			inode->i_rdev = raw_inode->i_block[0];
-		else for (block = 0; block < EXT2_N_BLOCKS; block++)
-			inode->u.ext2_i.i_data[block] = raw_inode->i_block[block];
-	}
+	inode->u.ext2_i.i_flags = raw_inode->i_flags;
+	inode->u.ext2_i.i_faddr = raw_inode->i_faddr;
+	inode->u.ext2_i.i_frag = raw_inode->i_frag;
+	inode->u.ext2_i.i_fsize = raw_inode->i_fsize;
+	inode->u.ext2_i.i_file_acl = raw_inode->i_file_acl;
+	inode->u.ext2_i.i_dir_acl = raw_inode->i_dir_acl;
+	inode->u.ext2_i.i_version = raw_inode->i_version;
+	inode->u.ext2_i.i_block_group = block_group;
+	inode->u.ext2_i.i_next_alloc_block = 0;
+	inode->u.ext2_i.i_next_alloc_goal = 0;
+	if (S_ISCHR(inode->i_mode) || S_ISBLK(inode->i_mode))
+		inode->i_rdev = raw_inode->i_block[0];
+	else for (block = 0; block < EXT2_N_BLOCKS; block++)
+		inode->u.ext2_i.i_data[block] = raw_inode->i_block[block];
 	brelse (bh);
 	inode->i_op = NULL;
-	if (S_ISREG(inode->i_mode))
+	if (inode->i_ino == EXT2_ACL_INO)
+		/* Nothing to do */ ;
+	else if (S_ISREG(inode->i_mode))
 		inode->i_op = &ext2_file_inode_operations;
 	else if (S_ISDIR(inode->i_mode))
 		inode->i_op = &ext2_dir_inode_operations;
@@ -735,20 +754,18 @@ void ext2_write_inode (struct inode * inode)
 	raw_inode->i_ctime = inode->i_ctime;
 	raw_inode->i_mtime = inode->i_mtime;
 	raw_inode->i_blocks = inode->i_blocks;
-	if (!S_ISFIFO(inode->i_mode)) {
-		raw_inode->i_dtime = inode->u.ext2_i.i_dtime;
-		raw_inode->i_flags = inode->u.ext2_i.i_flags;
-		raw_inode->i_faddr = inode->u.ext2_i.i_faddr;
-		raw_inode->i_frag = inode->u.ext2_i.i_frag;
-		raw_inode->i_fsize = inode->u.ext2_i.i_fsize;
-		raw_inode->i_file_acl = inode->u.ext2_i.i_file_acl;
-		raw_inode->i_dir_acl = inode->u.ext2_i.i_dir_acl;
-		raw_inode->i_version = inode->u.ext2_i.i_version;
-		if (S_ISCHR(inode->i_mode) || S_ISBLK(inode->i_mode))
-			raw_inode->i_block[0] = inode->i_rdev;
-		else for (block = 0; block < EXT2_N_BLOCKS; block++)
-			raw_inode->i_block[block] = inode->u.ext2_i.i_data[block];
-	}
+	raw_inode->i_dtime = inode->u.ext2_i.i_dtime;
+	raw_inode->i_flags = inode->u.ext2_i.i_flags;
+	raw_inode->i_faddr = inode->u.ext2_i.i_faddr;
+	raw_inode->i_frag = inode->u.ext2_i.i_frag;
+	raw_inode->i_fsize = inode->u.ext2_i.i_fsize;
+	raw_inode->i_file_acl = inode->u.ext2_i.i_file_acl;
+	raw_inode->i_dir_acl = inode->u.ext2_i.i_dir_acl;
+	raw_inode->i_version = inode->u.ext2_i.i_version;
+	if (S_ISCHR(inode->i_mode) || S_ISBLK(inode->i_mode))
+		raw_inode->i_block[0] = inode->i_rdev;
+	else for (block = 0; block < EXT2_N_BLOCKS; block++)
+		raw_inode->i_block[block] = inode->u.ext2_i.i_data[block];
 	bh->b_dirt = 1;
 	inode->i_dirt = 0;
 	brelse (bh);
