@@ -22,6 +22,8 @@
  * SCSI_TIMER		scsi.c timeout timer
  *
  * NET_TIMER		tcp/ip timeout timer
+ *
+ * COPRO_TIMER		387 timeout for buggy hardware..
  */
 
 #define BLANK_TIMER	0
@@ -33,6 +35,7 @@
 #define SCSI_TIMER 	18
 #define NET_TIMER	19
 #define SOUND_TIMER	20
+#define COPRO_TIMER	21
 
 struct timer_struct {
 	unsigned long expires;
