@@ -182,7 +182,7 @@ static int hung_up_tty_select(struct inode * inode, struct file * filp, int sel_
 }
 
 static int hung_up_tty_ioctl(struct inode * inode, struct file * file,
-			     unsigned int cmd, unsigned int arg)
+			     unsigned int cmd, unsigned long arg)
 {
 	return -EIO;
 }
@@ -243,6 +243,8 @@ void do_tty_hangup(struct tty_struct * tty, struct file_operations *fops)
 			continue;
 		if (filp->f_rdev != dev)
 			continue;
+		if (filp->f_inode && filp->f_inode->i_rdev == 0x0400)
+			continue;
 		if (filp->f_op != &tty_fops)
 			continue;
 		filp->f_op = fops;
@@ -543,12 +545,24 @@ void copy_to_cooked(struct tty_struct * tty)
 		if (c == __DISABLED_CHAR)
 			tty->lnext = 1;
 		if (L_CANON(tty) && !tty->lnext) {
-			if (c == KILL_CHAR(tty)) {
+			if (c == KILL_CHAR(tty) || c == WERASE_CHAR(tty)) {
+				int seen_alnums =
+				  (c == WERASE_CHAR(tty)) ? 0 : -1;
+
 				/* deal with killing the input line */
 				while(!(EMPTY(&tty->secondary) ||
 					(c=LAST(&tty->secondary))==10 ||
 					((EOF_CHAR(tty) != __DISABLED_CHAR) &&
 					 (c==EOF_CHAR(tty))))) {
+					/* if killing just a word, kill all
+					   non-alnum chars, then all alnum
+					   chars.  */
+					if (seen_alnums >= 0) {
+						if (isalnum(c))
+							seen_alnums++;
+						else if (seen_alnums)
+							break;
+					}
 					if (L_ECHO(tty)) {
 						if (c<32) {
 							put_tty_queue(8, &tty->write_q);
@@ -596,6 +610,10 @@ void copy_to_cooked(struct tty_struct * tty)
 			        tty->status_changed = 1;
 				tty->ctrl_status |= TIOCPKT_STOP;
 				tty->stopped=1;
+				if (IS_A_CONSOLE(tty->line)) {
+					set_vc_kbd_flag(kbd_table + fg_console, VC_SCROLLOCK);
+					set_leds();
+				}
 				continue;
 			}
 			if (((I_IXANY(tty)) && tty->stopped) ||
@@ -603,6 +621,10 @@ void copy_to_cooked(struct tty_struct * tty)
 			        tty->status_changed = 1;
 				tty->ctrl_status |= TIOCPKT_START;
 				tty->stopped=0;
+				if (IS_A_CONSOLE(tty->line)) {
+					clr_vc_kbd_flag(kbd_table + fg_console, VC_SCROLLOCK);
+					set_leds();
+				}
 				continue;
 			}
 		}
@@ -618,8 +640,10 @@ void copy_to_cooked(struct tty_struct * tty)
 				continue;
 			}
 			if (c == SUSPEND_CHAR(tty)) {
-				if (!is_orphaned_pgrp(tty->pgrp))
+				if (!is_orphaned_pgrp(tty->pgrp)) {
 					kill_pg(tty->pgrp, SIGTSTP, 1);
+					flush_input(tty);
+				}
 				continue;
 			}
 		}
@@ -1043,7 +1067,7 @@ end_init:
 	if (tty)
 		free_page((unsigned long) tty);
 	if (o_tty)
-		free_page((unsigned long) tty);
+		free_page((unsigned long) o_tty);
 	if (tp)
 		kfree_s(tp, sizeof(struct termios));
 	if (o_tp)
@@ -1060,6 +1084,7 @@ static void release_dev(int dev, struct file * filp)
 {
 	struct tty_struct *tty, *o_tty;
 	struct termios *tp, *o_tp;
+	struct task_struct **p;
 
 	tty = tty_table[dev];
 	tp = tty_termios[dev];
@@ -1107,6 +1132,15 @@ static void release_dev(int dev, struct file * filp)
 	if (tty->count)
 		return;
 
+	/*
+	 * Make sure there aren't any processes that still think this
+	 * tty is their controlling tty.
+	 */
+	for (p = &LAST_TASK ; p > &FIRST_TASK ; --p) {
+		if ((*p) && (*p)->tty == tty->line)
+		(*p)->tty = -1;
+	}
+
 	if (ldiscs[tty->disc].close != NULL)
 		ldiscs[tty->disc].close(tty);
 
